@@ -26,6 +26,7 @@ _sharingUrl = "https://drive.google.com/open?id="
 _INVALID_PATH_CHARS = ":\0"
 _log = getLogger("fs.googledrivefs")
 _rootMetadata = {"id": "root", "mimeType": _folderMimeType}
+_ALL_FIELDS = "id,mimeType,kind,name,createdTime,modifiedTime,size,permissions,appProperties,contentHints,md5Checksum"
 
 def _Escape(name):
 	name = name.replace("\\", "\\\\")
@@ -139,12 +140,13 @@ class SubGoogleDriveFS(SubFS):
 class GoogleDriveFS(FS):
 	subfs_class = SubGoogleDriveFS
 
-	def __init__(self, credentials):
+	def __init__(self, credentials, rootId=None):
 		super().__init__()
 
 		self.drive = build("drive", "v3", credentials=credentials, cache_discovery=False)
 		self.retryCount = 3
 		self.enforceSingleParent = False
+		self.rootId = rootId
 
 		_meta = self._meta = {
 			"case_insensitive": True, # it will even let you have 2 identical filenames in the same directory! But the search is case-insensitive
@@ -162,11 +164,10 @@ class GoogleDriveFS(FS):
 	def search(self, condition):
 		_log.info(f"search: {condition()}")
 		rawResults = self._fileQuery(condition())
-		# TODO - take account of paging
 		return (self._infoFromMetadata(x) for x in rawResults)
 
 	def _fileQuery(self, query):
-		allFields = "nextPageToken,files(id,mimeType,kind,name,createdTime,modifiedTime,size,permissions,appProperties,contentHints,md5Checksum)"
+		allFields = f"nextPageToken,files({_ALL_FIELDS})"
 		response = self.drive.files().list(q=query, fields=allFields).execute(num_retries=self.retryCount)
 		result = response["files"]
 		while "nextPageToken" in response:
@@ -175,13 +176,10 @@ class GoogleDriveFS(FS):
 		return result
 
 	def _childByName(self, parentId, childName):
-		# this "name=" clause seems to be case-insensitive, which means it's easier to model this
-		# as a case-insensitive filesystem
 		if not parentId:
 			parentId = "root"
-			# Google drive seems to somehow distinguish it's real root folder from folder named "root" in root folder.
-		query = f"trashed=False and name='{_Escape(childName)}' and '{parentId}' in parents"
-		result = self._fileQuery(query)
+		# this "name=" clause seems to be case-insensitive, which means it's easier to model this as a case-insensitive filesystem
+		result = self._fileQuery(f"trashed=False and name='{_Escape(childName)}' and '{parentId}' in parents")
 		if len(result) not in [0, 1]:
 			# Google drive doesn't follow the model of a filesystem, really
 			# but since most people will set it up to follow the model, we'll carry on regardless
@@ -190,15 +188,23 @@ class GoogleDriveFS(FS):
 		return result[0] if len(result) == 1 else None
 
 	def _childrenById(self, parentId):
-		query = f"trashed=False and '{parentId}' in parents"
-		return self._fileQuery(query)
+		return self._fileQuery(f"trashed=False and '{parentId}' in parents")
 
 	def _itemsFromPath(self, path):
 		pathIdMap = {"": _rootMetadata}
 		ipath = iteratepath(path)
 
 		pathSoFar = ""
-		parentId = None
+		parentId = self.rootId
+
+		if self.rootId is not None:
+			# if we have been given a `rootId` then get the info for this directory and set it as
+			# the root directory's metadata.
+			rootMetadata = self.drive.files().get(fileId=self.rootId, fields=_ALL_FIELDS).execute()
+			if rootMetadata is None:
+				return pathIdMap
+			pathIdMap[""] = rootMetadata
+
 		for childName in ipath:
 			pathSoFar = join(pathSoFar, childName)
 			metadata = self._childByName(parentId, childName)
@@ -211,22 +217,12 @@ class GoogleDriveFS(FS):
 		return pathIdMap
 
 	def _itemFromPath(self, path):
-		_log.debug(f"_itemFromPath: {path}")
-		ipath = iteratepath(path)
-
-		metadata = _rootMetadata
-		for childName in ipath:
-			parentId = metadata["id"] # pylint: disable=unsubscriptable-object
-			metadata = self._childByName(parentId, childName)
-			if metadata is None:
-				break
-
-		_log.debug(f"_itemFromPath -> {metadata}")
-		return metadata
+		pathIdMap = self._itemsFromPath(path)
+		return pathIdMap.get(path)
 
 	def _infoFromMetadata(self, metadata):  # pylint: disable=no-self-use
-		isRoot = isinstance(metadata, list) or metadata == _rootMetadata
-		isFolder = isRoot or (metadata["mimeType"] == _folderMimeType)
+		isRoot = (metadata == _rootMetadata)
+		isFolder = (metadata["mimeType"] == _folderMimeType)
 		rfc3339 = "%Y-%m-%dT%H:%M:%S.%fZ"
 		permissions = metadata.get("permissions", None)
 		rawInfo = {
@@ -243,7 +239,7 @@ class GoogleDriveFS(FS):
 				"type": ResourceType.directory if isFolder else ResourceType.file
 			},
 			"sharing": {
-				"id": None if isRoot else metadata["id"],
+				"id": metadata["id"],
 				"permissions": permissions,
 				"is_shared": len(permissions) > 1 if permissions is not None else None
 			}
@@ -265,7 +261,7 @@ class GoogleDriveFS(FS):
 		path = _CheckPath(path)
 		with self._lock:
 			metadata = self._itemFromPath(path)
-			if metadata is None or isinstance(metadata, list):
+			if metadata is None:
 				raise ResourceNotFound(path=path)
 			return self._infoFromMetadata(metadata)
 
@@ -273,7 +269,7 @@ class GoogleDriveFS(FS):
 		path = _CheckPath(path)
 		with self._lock:
 			metadata = self._itemFromPath(path)
-			if metadata is None or isinstance(metadata, list):
+			if metadata is None:
 				raise ResourceNotFound(path=path)
 			updatedData = {}
 			for namespace in info:
@@ -301,7 +297,7 @@ class GoogleDriveFS(FS):
 		path = _CheckPath(path)
 		with self._lock:
 			metadata = self._itemFromPath(path)
-			if metadata is None or isinstance(metadata, list):
+			if metadata is None:
 				raise ResourceNotFound(path=path)
 			if role not in ("reader", "writer", "commenter", "fileOrganizer", "organizer", "owner"):
 				raise OperationFailed(path=path, msg=f"unknown sharing role: {role}")
@@ -355,7 +351,7 @@ class GoogleDriveFS(FS):
 			if childMetadata is not None:
 				if recreate is False:
 					raise DirectoryExists(path=path)
-				return SubFS(self, path)
+				return SubGoogleDriveFS(self, path)
 
 			return self._createSubdirectory(basename(path), path, [parentMetadata["id"]])
 
